@@ -25,6 +25,7 @@
 #include <stdlib.h>
 
 #include <rest/oauth2-proxy.h>
+#include <libsoup/soup-gnome.h>
 #include <webkit/webkit.h>
 #include <json-glib/json-glib.h>
 
@@ -390,6 +391,7 @@ get_tokens_sync (GoaOAuth2Provider  *provider,
   gchar *ret_refresh_token;
   const gchar *payload;
   gsize payload_length;
+  const gchar *client_secret;
 
   ret = NULL;
   ret_access_token = NULL;
@@ -398,24 +400,26 @@ get_tokens_sync (GoaOAuth2Provider  *provider,
 
   proxy = rest_proxy_new (goa_oauth2_provider_get_token_uri (provider), FALSE);
   call = rest_proxy_new_call (proxy);
-  rest_proxy_call_set_method (call, "POST");
-  rest_proxy_call_add_header (call, "Content-Type", "application/x-www-form-urlencoded");
+
+  rest_proxy_call_set_method (call, "GET");
+  rest_proxy_call_add_param (call, "client_id", goa_oauth2_provider_get_client_id (provider));
+  rest_proxy_call_add_param (call, "redirect_uri", goa_oauth2_provider_get_redirect_uri (provider));
+
+  client_secret = goa_oauth2_provider_get_client_secret (provider);
+  if (client_secret != NULL)
+    rest_proxy_call_add_param (call, "client_secret", client_secret);
+
   if (refresh_token != NULL)
     {
       /* Swell, we have a refresh code - just use that */
-      rest_proxy_call_add_param (call, "client_id", goa_oauth2_provider_get_client_id (provider));
-      rest_proxy_call_add_param (call, "client_secret", goa_oauth2_provider_get_client_secret (provider));
       rest_proxy_call_add_param (call, "grant_type", "refresh_token");
       rest_proxy_call_add_param (call, "refresh_token", refresh_token);
     }
   else
     {
       /* No refresh code.. request an access token using the authorization code instead */
-      rest_proxy_call_add_param (call, "client_id", goa_oauth2_provider_get_client_id (provider));
-      rest_proxy_call_add_param (call, "client_secret", goa_oauth2_provider_get_client_secret (provider));
       rest_proxy_call_add_param (call, "grant_type", "authorization_code");
       rest_proxy_call_add_param (call, "code", authorization_code);
-      rest_proxy_call_add_param (call, "redirect_uri", goa_oauth2_provider_get_redirect_uri (provider));
     }
 
   /* TODO: cancellable support? */
@@ -465,13 +469,11 @@ get_tokens_sync (GoaOAuth2Provider  *provider,
     }
   else
     {
-      GError *local_error;
       JsonParser *parser;
       JsonObject *object;
 
-      local_error = NULL;
       parser = json_parser_new ();
-      if (!json_parser_load_from_data (parser, payload, payload_length, &local_error))
+      if (!json_parser_load_from_data (parser, payload, payload_length, error))
         {
           g_prefix_error (error, _("Error parsing response as JSON: "));
           g_object_unref (parser);
@@ -683,10 +685,13 @@ get_tokens_and_identity (GoaOAuth2Provider  *provider,
       SoupSession *webkit_soup_session;
       SoupCookieJar *cookie_jar;
 
+      webkit_soup_session = webkit_get_default_session ();
+      /* Get the proxy configuration from the GNOME settings */
+      soup_session_add_feature_by_type (webkit_soup_session, SOUP_TYPE_PROXY_RESOLVER_GNOME);
+
       /* Ensure we use an empty non-persistent cookie to avoid login
        * credentials being reused...
        */
-      webkit_soup_session = webkit_get_default_session ();
       soup_session_remove_feature_by_type (webkit_soup_session, SOUP_TYPE_COOKIE_JAR);
       cookie_jar = soup_cookie_jar_new ();
       soup_session_add_feature (webkit_soup_session, SOUP_SESSION_FEATURE (cookie_jar));
@@ -741,7 +746,7 @@ get_tokens_and_identity (GoaOAuth2Provider  *provider,
                                        &data.refresh_token,
                                        &data.access_token_expires_in,
                                        NULL, /* GCancellable */
-                                       error);
+                                       &data.error);
   if (data.access_token == NULL)
     {
       g_prefix_error (&data.error, _("Error getting an Access Token: "));
@@ -753,7 +758,7 @@ get_tokens_and_identity (GoaOAuth2Provider  *provider,
                                                          data.access_token,
                                                          &data.presentation_identity,
                                                          NULL, /* TODO: GCancellable */
-                                                         error);
+                                                         &data.error);
   if (data.identity == NULL)
     {
       g_prefix_error (&data.error, _("Error getting identity: "));
@@ -1333,6 +1338,10 @@ goa_oauth2_provider_build_object (GoaProvider         *provider,
     goto out;
 
   oauth2_based = goa_oauth2_based_skeleton_new ();
+  goa_oauth2_based_set_client_id (oauth2_based,
+                                    goa_oauth2_provider_get_client_id (GOA_OAUTH2_PROVIDER (provider)));
+  goa_oauth2_based_set_client_secret (oauth2_based,
+                                       goa_oauth2_provider_get_client_secret (GOA_OAUTH2_PROVIDER (provider)));
   /* Ensure D-Bus method invocations run in their own thread */
   g_dbus_interface_skeleton_set_flags (G_DBUS_INTERFACE_SKELETON (oauth2_based),
                                        G_DBUS_INTERFACE_SKELETON_FLAGS_HANDLE_METHOD_INVOCATIONS_IN_THREAD);
