@@ -30,6 +30,7 @@
 #include "goafacebookprovider.h"
 #include "goayahooprovider.h"
 #include "goatwitterprovider.h"
+#include "goawindowsliveprovider.h"
 
 #include "goaeditablelabel.h"
 
@@ -597,6 +598,10 @@ ensure_ep_and_builtins (void)
       type = GOA_TYPE_FACEBOOK_PROVIDER;
 #endif
 
+#ifdef GOA_WINDOWS_LIVE_ENABLED
+      type = GOA_TYPE_WINDOWS_LIVE_PROVIDER;
+#endif
+
       type = type; /* silence -Wunused-but-set-variable */
 
       g_once_init_leave (&once_init_value, 1);
